@@ -4,7 +4,7 @@
 Author: Liu Kun && 16031215@qq.com
 Date: 2024-11-01 10:31:09
 LastEditors: Liu Kun && 16031215@qq.com
-LastEditTime: 2024-11-03 15:37:34
+LastEditTime: 2024-11-03 18:57:41
 FilePath: \\Python\\My_Funcs\\OAFuncs\\OAFuncs\\oa_down\\hycom_3hourly.py
 Description:
 EditPlatform: vscode
@@ -16,13 +16,13 @@ import datetime
 import os
 import time
 import warnings
+from concurrent.futures import ThreadPoolExecutor
 from pathlib import Path
 
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import requests
-import xarray as xr
 from rich import print
 from rich.progress import Progress
 
@@ -51,6 +51,9 @@ data_info['hourly']['dataset']['GLBy0.08']['version'] = {'93.0': {}}
 # info details
 # time range
 # GLBv0.08
+# 在网页上提交超过范围的时间，会返回该数据集实际时间范围，从而纠正下面的时间范围
+# 目前只纠正了GLBv0.08 93.0的时间范围，具体到小时了
+# 其他数据集的时刻暂时默认为00起，21止
 data_info['hourly']['dataset']['GLBv0.08']['version']['53.X']['time_range'] = {
     'time_start': '19940101', 'time_end': '20151230'}
 data_info['hourly']['dataset']['GLBv0.08']['version']['56.3']['time_range'] = {
@@ -64,7 +67,7 @@ data_info['hourly']['dataset']['GLBv0.08']['version']['57.7']['time_range'] = {
 data_info['hourly']['dataset']['GLBv0.08']['version']['92.9']['time_range'] = {
     'time_start': '20171001', 'time_end': '20171231'}
 data_info['hourly']['dataset']['GLBv0.08']['version']['93.0']['time_range'] = {
-    'time_start': '20180101', 'time_end': '20200218'}
+    'time_start': '2018010112', 'time_end': '2020021909'}
 # GLBu0.08
 data_info['hourly']['dataset']['GLBu0.08']['version']['93.0']['time_range'] = {
     'time_start': '20180919', 'time_end': '20181208'}
@@ -343,38 +346,54 @@ def get_query_dict_full_depth(var, lon_min, lon_max, lat_min, lat_max, time_str_
     return query_dict
 
 
-def ymd_in_which_dataset_and_version(time_ymd):
-    time_ymd = int(time_ymd)
+def ymd_in_which_dataset_and_version(time_ymdh):
+    time_ymdh = int(time_ymdh)
     d_list = []
     v_list = []
     trange_list = []
+    have_data = False
     for dataset_name in data_info['hourly']['dataset'].keys():
         for version_name in data_info['hourly']['dataset'][dataset_name]['version'].keys():
             # print(data_info['hourly']['dataset'][dataset_name]
             #       ['version'][version_name]['time_range'].values())
             time_s, time_e = list(
                 data_info['hourly']['dataset'][dataset_name]['version'][version_name]['time_range'].values())
-            if time_ymd >= int(time_s) and time_ymd <= int(time_e):
+            time_s, time_e = str(time_s), str(time_e)
+            if len(time_s) == 8:
+                time_s = time_s + '00'
+            if len(time_e) == 8:
+                time_e = time_e + '21'
+            if time_ymdh >= int(time_s) and time_ymdh <= int(time_e):
                 d_list.append(dataset_name)
                 v_list.append(version_name)
                 trange_list.append(f'{time_s}-{time_e}')
-    print(f'[bold red]{time_ymd} is in the following dataset and version:')
-    for d, v, trange in zip(d_list, v_list, trange_list):
-        # print(f'{time_ymd} is in {d} {v} {trange}')
-        print(f'[bold blue]{d} {v} {trange}')
+                have_data = True
+    print(f'[bold red]{time_ymdh} is in the following dataset and version:')
+    if have_data:
+        for d, v, trange in zip(d_list, v_list, trange_list):
+            # print(f'{time_ymd} is in {d} {v} {trange}')
+            print(f'[bold blue]{d} {v} {trange}')
+    else:
+        raise ValueError(
+            f'[bold red]{time_ymdh} is not in any dataset and version')
 
 
-def direct_choose_dataset_and_version(time_ymd):
-    time_ymd = int(time_ymd)
+def direct_choose_dataset_and_version(time_ymdh):
+    time_ymdh = int(time_ymdh)
     for dataset_name in data_info['hourly']['dataset'].keys():
         for version_name in data_info['hourly']['dataset'][dataset_name]['version'].keys():
             [time_s, time_e] = list(data_info['hourly']['dataset'][dataset_name]['version'][version_name]['time_range'].values(
             ))
             # print(time_s, time_e, time_ymd)
-            if time_ymd >= int(time_s) and time_ymd <= int(time_e):
+            time_s, time_e = str(time_s), str(time_e)
+            if len(time_s) == 8:
+                time_s = time_s + '00'
+            if len(time_e) == 8:
+                time_e = time_e + '21'
+            if time_ymdh >= int(time_s) and time_ymdh <= int(time_e):
                 print(
                     f'[bold purple]dataset: {dataset_name}, version: {version_name} is chosen')
-                # print(f'{time_ymd} is in {dataset_name} {version_name}')
+                # print(f'{time_ymdh} is in {dataset_name} {version_name}')
                 return dataset_name, version_name
 
 
@@ -422,13 +441,6 @@ def clear_existing_file(file_full_path):
         print(f'{file_full_path} has been removed')
 
 
-def query_ncfile_time(file_full_path):
-    ds = xr.open_dataset(file_full_path)
-    ncfile_time = ds['time'].values[0]
-    print(f'文件 {file_full_path} 的时间为：[bold #BC8F8F] {ncfile_time}')
-    ds.close()
-
-
 def dlownload_file(target_url, store_path, file_name):
     # 创建会话
     s = requests.Session()
@@ -454,7 +466,7 @@ def dlownload_file(target_url, store_path, file_name):
                 f.write(response.content)
             # print(f'\r文件 {filename} 下载成功', end="")
             print(f'[bold green]文件 {filename} 下载成功')
-            query_ncfile_time(filename)
+            # query_ncfile_time(filename) # 这个函数在linux上目前会出问题
             download_success = True
 
         except requests.exceptions.HTTPError as errh:
@@ -480,28 +492,28 @@ def check_hour_is_valid(ymdh_str):
 
 
 def direct_download(var, lon_min=0, lon_max=359.92, lat_min=-80, lat_max=90, download_time='2024083100', depth=None, level_num=None, store_path=None, dataset_name=None, version_name=None):
-    time_ymd_str = str(download_time)[:8]
+    download_time = str(download_time)
     if not check_hour_is_valid(download_time):
         print('Please ensure the hour is 00, 03, 06, 09, 12, 15, 18, 21')
         raise ValueError('The hour is invalid')
     if dataset_name is None and version_name is None:
         print('The dataset_name and version_name are None, so the dataset and version will be chosen according to the download_time.\nIf there is more than one dataset and version in the time range, the first one will be chosen.')
         print('If you wanna choose the dataset and version by yourself, please set the dataset_name and version_name together.')
-        ymd_in_which_dataset_and_version(time_ymd_str)
+        ymd_in_which_dataset_and_version(download_time)
         dataset_name, version_name = direct_choose_dataset_and_version(
-            time_ymd_str)
+            download_time)
     elif dataset_name is None and version_name is not None:
         print('Please ensure the dataset_name is not None')
         print('If you do not add the dataset_name, both the dataset and version will be chosen according to the download_time.')
-        ymd_in_which_dataset_and_version(time_ymd_str)
+        ymd_in_which_dataset_and_version(download_time)
         dataset_name, version_name = direct_choose_dataset_and_version(
-            time_ymd_str)
+            download_time)
     elif dataset_name is not None and version_name is None:
         print('Please ensure the version_name is not None')
         print('If you do not add the version_name, both the dataset and version will be chosen according to the download_time.')
-        ymd_in_which_dataset_and_version(time_ymd_str)
+        ymd_in_which_dataset_and_version(download_time)
         dataset_name, version_name = direct_choose_dataset_and_version(
-            time_ymd_str)
+            download_time)
     else:
         print('The dataset_name and version_name are both set by yourself.')
 
@@ -540,7 +552,16 @@ def convert_full_name_to_short_name(full_name):
     return False
 
 
-def download(var, time_s, time_e, lon_min=0, lon_max=359.92, lat_min=-80, lat_max=90,  depth=None, level=None, store_path=None, dataset_name=None, version_name=None):
+def download_task(var, time_str, lon_min, lon_max, lat_min, lat_max, depth, level, store_path, dataset_name, version_name):
+    '''
+    # 并行下载任务
+    # 这个函数是为了并行下载而设置的，是必须的，直接调用direct_download并行下载会出问题
+    '''
+    direct_download(var, lon_min, lon_max, lat_min, lat_max, time_str,
+                    depth, level, store_path, dataset_name, version_name)
+
+
+def download(var, time_s, time_e, lon_min=0, lon_max=359.92, lat_min=-80, lat_max=90,  depth=None, level=None, store_path=None, dataset_name=None, version_name=None, num_workers=None):
     '''
     Description:
     Download the data of single time or a series of time
@@ -558,6 +579,7 @@ def download(var, time_s, time_e, lon_min=0, lon_max=359.92, lat_min=-80, lat_ma
     store_path: str, the path to store the data, default is None
     dataset_name: str, the dataset name, default is None
     version_name: str, the version name, default is None
+    num_workers: int, the number of workers, default is None
 
     Returns:
     None
@@ -585,10 +607,20 @@ def download(var, time_s, time_e, lon_min=0, lon_max=359.92, lat_min=-80, lat_ma
         with Progress() as progress:
             task = progress.add_task(
                 "[cyan]Downloading...", total=len(time_list))
-            for time_str in time_list:
-                direct_download(var, lon_min, lon_max, lat_min, lat_max,
-                                time_str, depth, level, store_path, dataset_name, version_name)
-                progress.update(task, advance=1)
+            if num_workers is None or num_workers <= 1:
+                # 串行方式
+                for time_str in time_list:
+                    direct_download(var, lon_min, lon_max, lat_min, lat_max,
+                                    time_str, depth, level, store_path, dataset_name, version_name)
+                    progress.update(task, advance=1)
+            else:
+                # 并行方式
+                with ThreadPoolExecutor(max_workers=num_workers) as executor:
+                    futures = [executor.submit(download_task, var, time_str, lon_min, lon_max, lat_min, lat_max,
+                                               depth, level, store_path, dataset_name, version_name) for time_str in time_list]
+                    for future in futures:
+                        future.add_done_callback(
+                            lambda _: progress.update(task, advance=1))
     else:
         print('Please ensure the ymdh_time_s is less than the ymdh_time_e')
 
