@@ -4,7 +4,7 @@
 Author: Liu Kun && 16031215@qq.com
 Date: 2024-09-17 17:26:11
 LastEditors: Liu Kun && 16031215@qq.com
-LastEditTime: 2024-09-28 17:19:12
+LastEditTime: 2024-10-20 21:12:58
 FilePath: \\Python\\My_Funcs\\OAFuncs\\OAFuncs\\oa_draw.py
 Description:  
 EditPlatform: vscode
@@ -14,20 +14,23 @@ Python Version: 3.11
 '''
 
 
-
-import warnings
-from mpl_toolkits.axes_grid1 import make_axes_locatable
-from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
-import cartopy.feature as cfeature
-import cartopy.crs as ccrs
 import math
+import warnings
+
+import cartopy.crs as ccrs
+import cartopy.feature as cfeature
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 import numpy as np
 import xarray as xr
+from cartopy.mpl.ticker import LatitudeFormatter, LongitudeFormatter
+from mpl_toolkits.axes_grid1 import make_axes_locatable
+
+__all__ = ['create_gif', 'xy2lonlat', 'plot_contourf',
+           'plot_contourf_lonlat', 'plot_quiver', 'plot_contourf_cartopy']
+
 warnings.filterwarnings('ignore')
 
-__all__ = ['create_gif', 'xy2lonlat', 'plot_contourf', 'plot_contourf_lonlat', 'plot_quiver', 'plot_contourf_cartopy']
 
 # ** 将生成图片/已有图片制作成动图
 def create_gif(image_list: list, gif_name: str, duration=0.2):  # 制作动图，默认间隔0.2
@@ -51,6 +54,13 @@ def create_gif(image_list: list, gif_name: str, duration=0.2):  # 制作动图
 
 # ** 转化经/纬度刻度
 def xy2lonlat(xy, lonlat='lon', decimal=2):
+    '''
+    param        {*} xy : 经/纬度列表
+    param        {*} lonlat : 'lon' or 'lat'
+    param        {*} decimal : 小数位数
+    return       {*} 转化后的经/纬度列表
+    example     : xy2lonlat(x, lonlat='lon', decimal=2)
+    '''
     def format_longitude(x_list):
         out_list = []
         for x in x_list:
@@ -82,7 +92,7 @@ def xy2lonlat(xy, lonlat='lon', decimal=2):
 
 
 # ** 设置colorbar格式
-class MyFormatter(mpl.ticker.ScalarFormatter):
+class _MyFormatter(mpl.ticker.ScalarFormatter):
     def __init__(self, cticks, fmt=None, useOffset=True, useMathText=True):
         mpl.ticker.ScalarFormatter.__init__(self, useOffset, useMathText)
         self.cticks = cticks
@@ -163,7 +173,7 @@ def plot_contourf(pic_data, picname=None, c_map='rainbow', minmax=None, labels=N
     # print(class_cticks)
     cct = abs(class_cticks[class_cticks != 0])
     if (min(cct) < 1e-2) or max(cct) > 1e4:  # 判断是否需要采用科学计数法
-        cb.formatter = MyFormatter(class_cticks)
+        cb.formatter = _MyFormatter(class_cticks)
         cb.update_ticks()
 
     if labels is not None:
@@ -189,6 +199,15 @@ def plot_contourf(pic_data, picname=None, c_map='rainbow', minmax=None, labels=N
 
 # ** 画等高线图，带经纬度坐标轴
 def plot_contourf_lonlat(data, lon, lat, interval=5, picname=None, c_map='rainbow'):
+    '''
+    param        {*} data : 二维数据
+    param        {*} lon : 经度
+    param        {*} lat : 纬度
+    param        {*} interval : 经纬度间隔
+    param        {*} picname : 图片保存的文件名(含路径)
+    param        {*} c_map : 颜色映射，默认rainbow
+    return       {*} 无返回值
+    '''
     if len(lon.shape) == 2:
         lon = lon[0, :]
     if len(lat.shape) == 2:
@@ -225,6 +244,19 @@ def plot_contourf_lonlat(data, lon, lat, interval=5, picname=None, c_map='rainbo
 
 # ** 绘制矢量场
 def plot_quiver(u, v, lon, lat, picname=None, cmap='coolwarm', scale=0.25, width=0.002, x_space=5, y_space=5):
+    '''
+    param        {*} u : 二维数据
+    param        {*} v : 二维数据
+    param        {*} lon : 经度, 1D or 2D
+    param        {*} lat : 纬度, 1D or 2D
+    param        {*} picname : 图片保存的文件名(含路径)
+    param        {*} cmap : 颜色映射，默认coolwarm
+    param        {*} scale : 箭头的大小 / 缩小程度
+    param        {*} width : 箭头的宽度
+    param        {*} x_space : x轴间隔
+    param        {*} y_space : y轴间隔
+    return       {*} 无返回值
+    '''
     # 创建新的网格位置变量(lat_c, lon_c)
     if len(lon.shape) == 1 and len(lat.shape) == 1:
         lon_c, lat_c = np.meshgrid(lon, lat)
@@ -269,11 +301,24 @@ def plot_quiver(u, v, lon, lat, picname=None, cmap='coolwarm', scale=0.25, width
 
 
 def plot_contourf_cartopy(data, lon, lat, picname=None, cmap='rainbow', cn_fill_num=20, fig_size=(12, 9), title='Cartopy', land_color='green', ocean_color='lightgrey'):
+    '''
+    param        {*} data : 二维数据
+    param        {*} lon : 经度
+    param        {*} lat : 纬度
+    param        {*} picname : 图片保存的文件名(含路径)
+    param        {*} cmap : 颜色映射，默认rainbow
+    param        {*} cn_fill_num : 等值线数量
+    param        {*} fig_size : 图片大小，默认(12,9)
+    param        {*} title : 图片标题
+    param        {*} land_color : 陆地颜色
+    param        {*} ocean_color : 海洋颜色
+    return       {*} 无返回值
+    '''
     if len(lon.shape) == 2:
         lon = lon[0, :]
     if len(lat.shape) == 2:
         lat = lat[:, 0]
-    
+
     data_max = np.nanmax(data)
     data_min = np.nanmin(data)
     levels = np.linspace(data_min, data_max, cn_fill_num)
@@ -282,7 +327,7 @@ def plot_contourf_cartopy(data, lon, lat, picname=None, cmap='rainbow', cn_fill_
     fig = plt.figure(figsize=fig_size)
     proj = ccrs.PlateCarree()
     ax = fig.add_subplot(1, 1, 1, projection=proj)
-    
+
     ax.set_extent([lon[0], lon[-1], lat[0], lat[-1]], crs=proj)
 
     ax.add_feature(cfeature.LAND, facecolor=land_color, edgecolor="k")
@@ -358,6 +403,7 @@ def plot_contourf_cartopy(data, lon, lat, picname=None, cmap='rainbow', cn_fill_
         picname, bbox_inches='tight', dpi=600) if picname is not None else plt.show()
     plt.close()
 
+
 if __name__ == '__main__':
     # ** 绘制填色图
     data = np.random.randn(100, 100)
